@@ -726,14 +726,14 @@ SRXAFSCB_GetCE(struct rx_call *callp, long index, AFSDBCacheEntry *cep)
         break;
     }
     cep->states = 0;
-    if (scp->flags & CM_SCACHEFLAG_STATD)
+    if (cm_HaveCallback(scp))
         cep->states |= 1;
     if (scp->flags & CM_SCACHEFLAG_RO || scp->flags & CM_SCACHEFLAG_PURERO)
         cep->states |= 4;
     if (scp->fileType == CM_SCACHETYPE_MOUNTPOINT &&
-        scp->mountPointStringp[0])
+        scp->mpDataVersion == scp->dataVersion)
         cep->states |= 8;
-    if (scp->flags & CM_SCACHEFLAG_WAITING)
+    if (!osi_QIsEmpty(&scp->waitQueueH))
         cep->states |= 0x40;
     code = 0;
 
@@ -841,14 +841,14 @@ SRXAFSCB_GetCE64(struct rx_call *callp, long index, AFSDBCacheEntry64 *cep)
         break;
     }
     cep->states = 0;
-    if (scp->flags & CM_SCACHEFLAG_STATD)
+    if (cm_HaveCallback(scp))
         cep->states |= 1;
     if (scp->flags & CM_SCACHEFLAG_RO || scp->flags & CM_SCACHEFLAG_PURERO)
         cep->states |= 4;
     if (scp->fileType == CM_SCACHETYPE_MOUNTPOINT &&
-        scp->mountPointStringp[0])
+        scp->mpDataVersion == scp->dataVersion)
         cep->states |= 8;
-    if (scp->flags & CM_SCACHEFLAG_WAITING)
+    if (!osi_QIsEmpty(&scp->waitQueueH))
         cep->states |= 0x40;
     code = 0;
 
@@ -1576,8 +1576,10 @@ int cm_HaveCallback(cm_scache_t *scp)
         return (cm_data.fakeDirVersion == scp->dataVersion);
     }
 #endif
-    if (cm_readonlyVolumeVersioning &&
-        (scp->flags & CM_SCACHEFLAG_PURERO)) {
+    if (scp->cbServerp != NULL)
+	return 1;
+
+    if (scp->flags & CM_SCACHEFLAG_PURERO) {
         cm_volume_t *volp = cm_GetVolumeByFID(&scp->fid);
         if (volp) {
             int haveCB = 0;
@@ -1592,6 +1594,7 @@ int cm_HaveCallback(cm_scache_t *scp)
             }
             if (cm_readonlyVolumeVersioning &&
                 !haveCB &&
+                scp->volumeCreationDate != 0 &&
                 volp->creationDateRO == scp->volumeCreationDate &&
                 volp->cbServerpRO != NULL) {
                 haveCB = 1;
@@ -1600,10 +1603,7 @@ int cm_HaveCallback(cm_scache_t *scp)
             return haveCB;
         }
     }
-    if (scp->cbServerp != NULL)
-	return 1;
-    else
-        return 0;
+    return 0;
 }
 
 /* need to detect a broken callback that races with our obtaining a callback.
@@ -1742,7 +1742,10 @@ cm_EndCallbackGrantingCall(cm_scache_t *scp, cm_callbackRequest_t *cbrp,
                     lock_ObtainWrite(&cm_scacheLock);
                     volp->cbExpiresRO = scp->cbExpires;
                     volp->cbIssuedRO = scp->cbIssued;
-                    volp->creationDateRO = volSyncp->spare1;
+                    if (cm_readonlyVolumeVersioning || !(flags & CM_CALLBACK_BULKSTAT))
+                        volp->creationDateRO = volSyncp->spare1;
+                    else
+                        volp->creationDateRO = 0;
                     if (volp->cbServerpRO != scp->cbServerp) {
                         if (volp->cbServerpRO)
                             cm_PutServer(volp->cbServerpRO);
@@ -1862,7 +1865,7 @@ long cm_GetCallback(cm_scache_t *scp, struct cm_user *userp,
                                      &afsStatus, &callback, &volSync);
             rx_PutConnection(rxconnp);
 
-        } while (cm_Analyze(connp, userp, reqp, &sfid, 0, &volSync, NULL,
+        } while (cm_Analyze(connp, userp, reqp, &sfid, NULL, 0, &volSync, NULL,
                             &cbr, code));
         code = cm_MapRPCError(code, reqp);
         if (code)
