@@ -12,22 +12,15 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/rxkad/test/stress_c.c,v 1.9.14.1 2007/04/10 18:39:55 shadow Exp $");
+#include <roken.h>
 
 #include <afs/stds.h>
-#include <sys/types.h>
-#include <stdio.h>
-#ifdef AFS_NT40_ENV
-#include <winsock2.h>
-#else
-#include <netdb.h>
-#include <netinet/in.h>
-#endif
+
 #include <afs/com_err.h>
 #include <afs/afsutil.h>
 #include <rx/rxkad.h>
 #include <afs/auth.h>
+#include <des_prototypes.h>
 #include "stress.h"
 #include "stress_internal.h"
 #ifdef AFS_PTHREAD_ENV
@@ -40,10 +33,9 @@ extern int maxSkew;
 static char *whoami;
 
 static long
-GetServer(aname)
-     IN char *aname;
+GetServer(const char *aname)
 {
-    register struct hostent *th;
+    struct hostent *th;
     long addr;
 
     th = gethostbyname(aname);
@@ -56,11 +48,8 @@ GetServer(aname)
 }
 
 static long
-GetToken(versionP, session, ticketLenP, ticket, cell)
-     OUT long *versionP;
-     OUT struct ktc_encryptionKey *session;
-     OUT int *ticketLenP;
-     OUT char *ticket;
+GetToken(long *versionP, struct ktc_encryptionKey *session,
+	 int *ticketLenP, char *ticket, char *cell)
 {
     struct ktc_principal sname;
     struct ktc_token ttoken;
@@ -81,17 +70,14 @@ GetToken(versionP, session, ticketLenP, ticket, cell)
 }
 
 static long
-GetTicket(versionP, session, ticketLenP, ticket, cell)
-     OUT long *versionP;
-     OUT struct ktc_encryptionKey *session;
-     OUT int *ticketLenP;
-     OUT char *ticket;
+GetTicket(long *versionP, struct ktc_encryptionKey *session,
+	  int *ticketLenP, char *ticket, char *cell)
 {
     long code;
 
     /* create random session key, using key for seed to good random */
-    des_init_random_number_generator(&serviceKey);
-    code = des_random_key(session);
+    des_init_random_number_generator(ktc_to_cblock(&serviceKey));
+    code = des_random_key(ktc_to_cblock(session));
     if (code)
 	return code;
 
@@ -120,10 +106,7 @@ struct client {
 };
 
 static long
-Copious(c, buf, buflen)
-     IN struct client *c;
-     IN u_char *buf;
-     IN u_long buflen;
+Copious(struct client *c, char *buf, u_long buflen)
 {
     long code;
     struct rx_call *call;
@@ -132,7 +115,7 @@ Copious(c, buf, buflen)
     long outlen = c->recvLen;
     long d = 23;
     long mysum;
-    long outsum;
+    size_t outsum;
 
     mysum = 0;
     for (i = 0; i < inlen; i++)
@@ -194,12 +177,10 @@ Copious(c, buf, buflen)
 }
 
 static long
-DoClient(index, rock)
-     IN u_int index;
-     IN opaque rock;
+DoClient(int index, opaque rock)
 {
     struct client *c = (struct client *)rock;
-    long code;
+    long code = 0;
     int i;
     u_long n, inc_n;
 
@@ -226,7 +207,7 @@ DoClient(index, rock)
 
     if (c->copiousCalls[index] > 0) {
 	u_long buflen = 10000;
-	u_char *buf = (u_char *) osi_Alloc(buflen);
+	char *buf = osi_Alloc(buflen);
 	for (i = 0; i < c->copiousCalls[index]; i++) {
 	    code = Copious(c, buf, buflen);
 	    if (code)
@@ -244,7 +225,7 @@ struct worker {
     long exitCode;		/* is PROCESSRUNNING until exit */
     int index;
     opaque rock;
-    long (*proc) ();
+    long (*proc) (int, opaque);
 };
 
 #ifdef AFS_PTHREAD_ENV
@@ -260,10 +241,10 @@ WorkerInit(void)
 #endif
 static struct worker *workers;
 
-static long
-DoWorker(w)
-     IN struct worker *w;
+static void *
+DoWorker(void * rock)
 {
+    struct worker *w = rock;
     long code;
     code = (*w->proc) (w->index, w->rock);
 #ifdef AFS_PTHREAD_ENV
@@ -278,16 +259,13 @@ DoWorker(w)
 #else
     LWP_NoYieldSignal(&workers);
 #endif
-    return code;
+    return (void *)(intptr_t)code;
 }
 
 #define MAX_CTHREADS 25
 
 static long
-CallSimultaneously(threads, rock, proc)
-     IN u_int threads;
-     IN opaque rock;
-     IN long (*proc) ();
+CallSimultaneously(u_int threads, opaque rock, long (*proc)(int, opaque))
 {
     long code;
     int i;
@@ -381,10 +359,7 @@ CallSimultaneously(threads, rock, proc)
 }
 
 static void
-DivideUpCalls(calls, threads, threadCalls)
-     IN u_long calls;
-     IN u_int threads;
-     IN u_long threadCalls[];
+DivideUpCalls(u_long calls, u_int threads, u_long threadCalls[])
 {
     int i;
     for (i = 0; i < threads; i++) {
@@ -394,17 +369,15 @@ DivideUpCalls(calls, threads, threadCalls)
 }
 
 static double
-ftime()
+ftime(void)
 {
     struct timeval tv;
-    gettimeofday(&tv, 0);
+    gettimeofday(&tv, NULL);
     return (double)tv.tv_sec + (double)tv.tv_usec / 1000000.0;
 }
 
 static long
-RunLoadTest(parms, conn)
-     IN struct clientParms *parms;
-     IN struct rx_connection *conn;
+RunLoadTest(struct clientParms *parms, struct rx_connection *conn)
 {
     long code;
     struct client c;
@@ -438,7 +411,7 @@ RunLoadTest(parms, conn)
 	    parms->fastCalls + parms->slowCalls + parms->copiousCalls;
 	int t = (interval / totalCalls) * 1000.0 + 0.5;
 	if (totalCalls > 0) {
-	    printf("For %d calls: %d msec/call\n", totalCalls, t);
+	    printf("For %lu calls: %d msec/call\n", totalCalls, t);
 	}
 	if (parms->copiousCalls > 0) {
 	    long n = parms->sendLen + parms->recvLen;
@@ -460,7 +433,7 @@ RunLoadTest(parms, conn)
 	    }
 #endif
 	    printf
-		("For %d copious calls, %d send + %d recv = %d bytes each: %d kbytes/sec\n",
+		("For %lu copious calls, %lu send + %lu recv = %lu bytes each: %d kbytes/sec\n",
 		 parms->copiousCalls, parms->sendLen, parms->recvLen, n, b);
 #if 0
 	    printf("%g\n", kbps);
@@ -471,9 +444,7 @@ RunLoadTest(parms, conn)
 }
 
 static long
-RepeatLoadTest(parms, conn)
-     IN struct clientParms *parms;
-     IN struct rx_connection *conn;
+RepeatLoadTest(struct clientParms *parms, struct rx_connection *conn)
 {
     long code;
     long count;
@@ -530,14 +501,13 @@ struct multiChannel {
     int done;
     long *codes;
     int changes[RX_MAXCALLS];
-    long callNumbers[RX_MAXCALLS];
+    afs_int32 callNumbers[RX_MAXCALLS];
 };
 #define BIG_PRIME 1257056893	/* 0x4AED2A7d */
 static u_long sequence = 0;
 
 static long
-FastCall(conn)
-     IN struct rx_connection *conn;
+FastCall(struct rx_connection *conn)
 {
     long code;
     u_long n = (sequence = sequence * BIG_PRIME + BIG_PRIME);
@@ -552,13 +522,11 @@ FastCall(conn)
 }
 
 static long
-UniChannelCall(index, rock)
-     IN u_int index;
-     IN opaque rock;
+UniChannelCall(int index, opaque rock)
 {
     struct multiChannel *mc = (struct multiChannel *)rock;
     long code;
-    long callNumbers[RX_MAXCALLS];
+    afs_int32 callNumbers[RX_MAXCALLS];
     int unchanged;
 
     code = 0;
@@ -587,11 +555,8 @@ UniChannelCall(index, rock)
 }
 
 static long
-MakeMultiChannelCall(conn, each, expectedCode, codes)
-     IN struct rx_connection *conn;
-     IN int each;		/* calls to make on each channel */
-     IN long expectedCode;
-     OUT long codes[];
+MakeMultiChannelCall(struct rx_connection *conn, int each, 
+	             long expectedCode, long codes[])
 {
     long code;
     int i;
@@ -620,11 +585,8 @@ MakeMultiChannelCall(conn, each, expectedCode, codes)
 }
 
 static long
-CheckCallFailure(conn, codes, code, msg)
-     IN struct rx_connection *conn;
-     IN long codes[];
-     IN long code;
-     IN char *msg;
+CheckCallFailure(struct rx_connection *conn, long codes[], long code,
+		 char *msg)
 {
     if (code == 0) {
 	fprintf(stderr, "Failed to detect %s\n", msg);
@@ -653,7 +615,7 @@ CheckCallFailure(conn, codes, code, msg)
 		sprintf(buf, "connection dead following %s", msg);
 		code = FastCall(conn);
 		if (code)
-		    afs_com_err(whoami, code, buf);
+		    afs_com_err(whoami, code, "%s", buf);
 	    }
 	}
     }
@@ -663,11 +625,8 @@ CheckCallFailure(conn, codes, code, msg)
 #endif /* rx_GetPacketCksum */
 
 static long
-RunCallTest(parms, host, sc, si)
-     IN struct clientParms *parms;
-     IN long host;
-     IN struct rx_securityClass *sc;
-     IN long si;
+RunCallTest(struct clientParms *parms, long host,
+	    struct rx_securityClass *sc, long si)
 {
     long code;
 
@@ -683,7 +642,7 @@ RunCallTest(parms, host, sc, si)
     int i, ch;
     struct rx_connection *conn;
     long firstCall;
-    long callNumbers[RX_MAXCALLS];
+    afs_int32 callNumbers[RX_MAXCALLS];
     long codes[RX_MAXCALLS];
     long retCode = 0;		/* ret. if nothing fatal goes wrong */
 
@@ -879,9 +838,7 @@ static struct {
 #define IO_REDIRECTCHALLENGE	2
 
 static int
-HandleIncoming(p, addr)
-     INOUT struct rx_packet *p;
-     INOUT struct sockaddr_in *addr;
+HandleIncoming(struct rx_packet *p, struct sockaddr_in *addr)
 {
     int client;			/* packet sent by client */
     u_char type;		/* packet type */
@@ -935,9 +892,7 @@ static struct {
 #define OO_MUNGCKSUM	3
 
 static int
-HandleOutgoing(p, addr)
-     INOUT struct rx_packet *p;
-     INOUT struct sockaddr_in *addr;
+HandleOutgoing(struct rx_packet *p, struct sockaddr_in *addr)
 {
     int client;			/* packet sent by client */
     u_char type;		/* packet type */
@@ -1003,10 +958,10 @@ SlowCallInit(void)
 }
 #endif
 static long slowCallCode;
-static long
-SlowCall(conn)
-     IN opaque conn;
+static void *
+SlowCall(void * rock)
 {
+    struct rx_connection *conn = rock;
     u_long ntime;
     u_long now;
     long temp_rc;
@@ -1033,17 +988,14 @@ SlowCall(conn)
 #else
     LWP_NoYieldSignal(&slowCallCode);
 #endif
-    return temp_rc;
+    return (void *)(intptr_t)temp_rc;
 }
 
 #endif /* rx_GetPacketCksum */
 
 static long
-RunHijackTest(parms, host, sc, si)
-     IN struct clientParms *parms;
-     IN long host;
-     IN struct rx_securityClass *sc;
-     IN long si;
+RunHijackTest(struct clientParms *parms, long host,
+	      struct rx_securityClass *sc, long si)
 {
 
 #ifndef rx_GetPacketCksum
@@ -1055,9 +1007,6 @@ RunHijackTest(parms, host, sc, si)
 
 #else
 
-    extern int (*rx_justReceived) ();
-    extern int (*rx_almostSent) ();
-
     long code;
     struct rx_connection *conn = 0;
     struct rx_connection *otherConn = 0;
@@ -1271,8 +1220,7 @@ RunHijackTest(parms, host, sc, si)
 }
 
 long
-rxkst_StartClient(parms)
-     IN struct clientParms *parms;
+rxkst_StartClient(struct clientParms *parms)
 {
     long code;
     long host;
@@ -1284,7 +1232,7 @@ rxkst_StartClient(parms)
     host = GetServer(parms->server);
 
     if (parms->authentication >= 0) {
-	long kvno;
+	long kvno = 0;
 	char ticket[MAXKTCTICKETLEN];
 	int ticketLen;
 	struct ktc_encryptionKey Ksession;
@@ -1303,12 +1251,12 @@ rxkst_StartClient(parms)
 	    rxkad_NewClientSecurityObject(parms->authentication, &Ksession,
 					  kvno, ticketLen, ticket);
 	assert(sc);
-	scIndex = 2;		/* kerberos security index */
+	scIndex = RX_SECIDX_KAD;
     } else {
 	/* unauthenticated connection */
 	sc = rxnull_NewClientSecurityObject();
 	assert(sc);
-	scIndex = 0;		/* null security index */
+	scIndex = RX_SECIDX_NULL;
     }
 
     code = 0;
